from casadi import veccat, MX, vcat
import copy

keywords = {"x","u","z","p","c","y"}
for k in set(keywords):
  keywords.add("n"+k)
keywords.add("all")
keywords.add("ode")

class DotDict(object):
  """like a dict but access through . """
  def __init__(self, d=None):
    if d is None:
      d = {}
    self._d = d

  def __getattr__(self, k):
    try:
      return self._d[k]
    except:
      raise AttributeError()

  def __deepcopy__(self, memo):
    if id(self) in memo:
        return memo[id(self)]
    ret = self.__class__(copy.deepcopy(self._d))
    
    memo[id(self)] = ret
    return ret

  def _update(self, d, allow_keyword=False):
    for k,v in d.items():
      if allow_keyword:
        if k in self._d:
          old = self._d[k]
          if isinstance(old,list):
            self._d[k]=old+v
          else:
            self._d[k] = veccat(old,v)
        else:
          self._d[k] = v
      else:
        if k in keywords:
          raise Exception("'%s' is a reserved keyword. Please use another name." % k)
        if k in self._d:
          raise Exception("Name collision: '%s' already in use." % k)
        self._d[k] = v
      
  def __repr__(self,indent=0):
    s = "{\n"
    for k,v in sorted(self._d.items()):
      s += ("  " * (indent+1)) + k
      try:
        s += ": " + v.__repr__(indent=indent+1)
      except:
        if isinstance(v, list):
          s+= ": " + str(v)
        #s += str(v)
        pass

      s += "\n"
    s+=("  " * indent) + "}"
    return s


class Structure:
  def __init__(self, definition,prefix=""):
    if isinstance(definition,MX):
      self._symbols = [definition]
      self._concat = definition
      self._numel = definition
      self._names = [definition.name()]
    else:
      symbols = []
      for d in definition:
        size = d["size"] if "size" in d else 1
        name = d["name"]
        symbols.append(MX.sym(prefix+name, size))
      self._names = [d["name"] for d in definition]
      self._symbols = symbols
      self._concat = vcat(symbols)
      self._numel = self._concat.numel()

  def __MX__(self):
    return self._concat


def remove_prefix(n,prefix):
  if n.startswith(prefix):
    return n[len(prefix):]
  else:
    n


# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

import sphinx_rtd_theme

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath('../..'))


# -- Project information -----------------------------------------------------

project = 'impact'
copyright = '2021 - 2023, MECO Research Team, KU Leuven'
author = 'MECO Research Team, KU Leuven'

# The full version, including alpha/beta/rc tags
release = '0.0.1'


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
'sphinx.ext.autodoc',
'sphinx.ext.coverage',
'sphinx.ext.intersphinx',
'sphinx.ext.viewcode',
'sphinx.ext.todo',
'sphinx.ext.mathjax',
'sphinx_gallery.gen_gallery',
'sphinx_inline_tabs',
]

todo_include_todos = True

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

autodoc_default_options = {
    'special-members': '__init__',
    'exclude-members': '__weakref__'
}

# from sphinx_gallery.sorting import ExampleTitleSortKey

sphinx_gallery_conf = {
    'examples_dirs': '../../examples',   # path to your example scripts
    'gallery_dirs': 'examples',  # path where to save gallery generated examples
    # 'filename_pattern': '/',
    'filename_pattern': '/cart_pendulum.py',
    'ignore_pattern': '/cart_pendulum_|/double_integrator|/generate_models',
    # 'within_subsection_order': ExampleTitleSortKey,
    # 'plot_gallery': 'False',
    # 'binder': {
    #   'org': 'meco-software',
    #   'repo': 'rockit.git', # URL will be fixed in .gitlab-ci.yml
    #   'branch': 'master',
    #   'binderhub_url': 'https://mybinder.org',
    #   'dependencies': ['../../.binder/requirements.txt'],
    # }
}



# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# html_theme = 'alabaster'
html_theme = "sphinx_rtd_theme"

html_theme_options = {
    'logo_only': True,
    'prev_next_buttons_location': 'bottom',
    'style_external_links': False,
    # Toc options
    'collapse_navigation': False,
    'sticky_navigation': True,
    'navigation_depth': 4,
    'includehidden': True,
    'titles_only': False
}

html_logo = "../figures/logo/impact_logo.svg"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# Do not order alphabetically
autodoc_member_order = 'bysource'




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import glob
import os

version = "0.4.2"

print(find_packages(exclude=['tests', 'examples']))

setup(
    name='impact-meco',
    version=version,
    author="MECO-Group",
    author_email="joris.gillis@kuleuven.be",
    description="DIRAC MPC",
    license='LICENSE',
    url='https://gitlab.kuleuven.be/meco-software/impact',
    packages=find_packages(exclude=['tests', 'examples']),
    include_package_data=True,
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    install_requires=[
        'casadi>=3.5,<4.0',
        'pyyaml',
        'lxml',
        'rockit-meco>=0.6.0',
    ],
    download_url='https://gitlab.kuleuven.be/meco-software/impact/-/archive/v%s/impact-v%s.tar.gz' % (version, version)
)
